@@ -11,7 +11,8 @@ fi
 libdir="$1"
 private_libdir="$2"
 
-if [ ! -f "$private_libdir/libjulia-internal.so" ]; then
+if [ ! -f "$private_libdir/libjulia-internal.so" ] && \
+   [ ! -f "$private_libdir/libjulia-internal-debug.so" ]; then
     echo "ERROR: Could not open $private_libdir/libjulia-internal.so" >&2
     exit 2
 fi
@@ -24,7 +25,11 @@ find_shlib ()
 }
 
 # Discover libstdc++ location and name
-LIBSTD=$(find_shlib "$private_libdir/libjulia-internal.so" "libstdc++.so")
+if [ -f "$private_libdir/libjulia-internal.so" ]; then
+    LIBSTD=$(find_shlib "$private_libdir/libjulia-internal.so" "libstdc++.so")
+elif [ -f "$private_libdir/libjulia-internal-debug.so" ]; then
+    LIBSTD=$(find_shlib "$private_libdir/libjulia-internal-debug.so" "libstdc++.so")
+fi
 LIBSTD_NAME=$(basename $LIBSTD)
 LIBSTD_DIR=$(dirname $LIBSTD)
 
